import * as fs from 'node:fs/promises'

import yaml from 'js-yaml'

import { checkGitHubAPIAccess, checkGitHubSSHAccess } from '@liquid-labs/github-toolkit'
import { types } from '@liquid-labs/liq-credentials-db'

const setupCredentials = async({ credentialsDB }) => {
  credentialsDB.registerCredentialType({
    key         : 'GITHUB_SSH',
    name        : 'GitHub SSH key',
    description : 'Used to authenticate user for git operations such as clone, fetch, and push.',
    type        : types.SSH_KEY_PAIR,
    verifyFunc  : ({ files }) => checkGitHubAPIAccess({ filePath : files[0] })
  })
  credentialsDB.registerCredentialType({
    key          : 'GITHUB_API',
    name         : 'GitHub API token',
    description  : 'Used to authenticate REST/API actions.',
    type         : types.AUTH_TOKEN,
    verifyFunc   : ({ files }) => checkGitHubSSHAccess({ privKeyPath : files[0] }),
    getTokenFunc : async({ files }) => {
      const credContents = await fs.readFile(files[0], { encoding : 'utf8' })
      const credentialData = yaml.load(credContents)

      const token = credentialData?.['github.com']?.[0]?.oauth_token
      if (token === undefined) {
        throw new Error("The 'GITHUB_API' token was not defined in the credential source.")
      }

      return token
    }
  })
}

export { setupCredentials }
